#include "myqtapp.h"

#define CVX_RED CV_RGB(0xff,0x00,0x00)
#define CVX_GREEN CV_RGB(0x00,0xff,0x00)
#define CVX_BLUE CV_RGB(0x00,0x00,0xff)
#define CVX_YELLOW CV_RGB(0xff, 0xff, 0x00)


// Global w-curve parameters for trackbars.
int g_lambda_tb, g_width_tb, g_height_tb, g_rot_tb, g_xtrans_tb, g_ytrans_tb;
// Global value to determine the visibility of the input file in
// background of the output image.
int g_input_alpha;
// Threshold for trackbar.
int g_threshold;
// How much to cut off from bottom and top of the w-curve.
int g_cutoff_top;
int g_cutoff_bottom;

// Save the current energy value globally such that it can be returned quickly
// upon exit.
double g_energy = 0.0;


// Global variables storing the width, height and position of the bud
// contour. This can help the optimiser to know its minima, maxima and
// starting points.
int g_bud_width;
int g_bud_height;
int g_bud_pos_x;
int g_bud_pos_y;

// To store the actual pixel width of the w-curve.
int g_w_curve_pixel_width;

// This is the value of t which yields the maximum width for the
// current w-curve. (See below...)
double g_rightmost_t;

// Two flags to indicate if the mous callback function should read the
// coordinates.
bool g_read_top, g_read_bottom;
// To store the top coordinates until the bottom ones are set.
int g_mouse_set_top_x, g_mouse_set_top_y;

// The values for the region of interest.
int g_roi_top_x, g_roi_top_y, g_roi_width, g_roi_height;

// A flag to indicate which side was changed in resize.
// This is important for getting the proper values from
// the mouse pointer.
bool g_resized_width;

// A flag to indicate whether the extra image of the distance
// transformation should be shown.
bool g_show_dist_image = false;

// The same for the binary threshold image.
bool g_show_binary_image = false;

// Global pointers to bud contour and output image.
// (Needed by the trackbar-function which cannot have any arguments...)
CvSeq* g_contour_bud = NULL;
IplImage* g_output_image = NULL;
IplImage* g_input_image = NULL;
IplImage* g_resized_image = NULL;
IplImage* g_cropped_image = NULL;
IplImage* g_deviation_image = NULL;

// Dimension of the deviation image.
int g_deviation_width = 0;
int g_deviation_height = 0;

// Global pointers to swap data of the energy funciton.
IplImage* g_gray_image = NULL;
IplImage* g_dist_image = NULL;
IplImage* g_resized_dist_image = NULL;
IplImage* g_resized_binary_image = NULL;

// Global storage for the w_curve which is often recalculated.
CvMemStorage* g_storage_w_left = NULL;
CvMemStorage* g_storage_w_right = NULL;
// Global storage for the bud curve beacause on_tb_thresh needs also
// access to it.
CvMemStorage* g_storage_bud = NULL;





// A function to increase or decrease degrees such that 0=360.
int add_degree(int left, int right) {
	int result = left+right;
	while (result < 0) {
		result += 360;
	}
	return result % 360;
}


// Create a w-curve contour according to the parameters.  We want to
// ignore the bottom and the top. Hence, we need two sequences: One
// for the left and one for the right part of the curve. The sequences
// are stored in "storage_left" and "storage_right" and their pointers
// are returned "seq_left" and "seq_right".
void create_w_curve(
	double lambda,
	double width, double height,
	double rot,
	double xtrans, double ytrans, 
	CvSeq* &seq_left, CvSeq* &seq_right,
	CvMemStorage* storage_left, CvMemStorage* storage_right
) {


	// Clear storage.
	cvClearMemStorage(storage_left);
	cvClearMemStorage(storage_right);

	// Create empty sequences to store the w-curve contour.
	seq_left = cvCreateSeq(CV_SEQ_ELTYPE_POINT |   // Some flags...
		CV_SEQ_KIND_CURVE,
		sizeof(CvSeq),   // header size
		sizeof(CvPoint), // elemSize
		storage_left);   // storage

	seq_right = cvCreateSeq(CV_SEQ_ELTYPE_POINT |   // Some flags...
		CV_SEQ_KIND_CURVE,
		sizeof(CvSeq),   // header size
		sizeof(CvPoint), // elemSize
		storage_right);  // storage


	// Point object to be inserted in the curve sequencs.
	CvPoint nextPoint;

	// Pointer for the return value of cvSeqPush.
	CvPoint* added;

	// For rotation according to angle.
	rot = M_PI*rot/180;

	// Variables to be reused during the following loop.
	double coeff;
	int other_x, other_y;
	// Swap variable for rotation.
	int tmp_x;

	// Remember the rightmost x-coordinate to calculate the actual width
	// of the curve in pixels.
	int rightmost = 0;

	// Go through the t parameter of the W-curve function.
	for (double t=-20; t<20; t += 0.05) {

		// Calculate x and y coordinates according to lambda, height and
		// width. The "other values" become the coordinates of the
		// axis-symmetric points.
		coeff = exp(-lambda*t) + height*exp(t)/2;
		coeff = 1/coeff;
		nextPoint.x = coeff * width;
		nextPoint.y = coeff * pow(height, 2) * exp(t) /2 ;
		other_x = -nextPoint.x;
		other_y = nextPoint.y;

		// Remember the rightmost.
		// And remember the corresponding t.
		// I don't know how to mathmatically calculate the minimum t for
		// e^(-t*lambda) + (h/2)*e^t .
		if (nextPoint.x > rightmost) {
			rightmost = nextPoint.x;
			g_rightmost_t = t;
		}
		
		// We do not want the top and bottom: Cut-offs!
		if ((nextPoint.y < height*((double)g_cutoff_top/100)) ||
				(nextPoint.y > height*(1-((double)g_cutoff_bottom/100)))
				) continue;
		
		// Rotation should be around the center of the curve, instead of
		// arround the top or bottom. Hence move the curve half its height
		// "down". This has also the effect that the height will grow in
		// both directions.
		nextPoint.y -= height/2;
		other_y -= height/2;
		
		// Rotate.
		tmp_x = nextPoint.x * cos(rot) - nextPoint.y * sin(rot);
		nextPoint.y = nextPoint.x * sin(rot) + nextPoint.y * cos(rot);
		nextPoint.x = tmp_x;
		tmp_x = other_x * cos(rot) - other_y * sin(rot);
		other_y = other_x * sin(rot) + other_y * cos(rot);
		other_x = tmp_x;
		
		// Translation.
		nextPoint.x += (int) xtrans;
		nextPoint.y += (int) ytrans;
		other_x += (int) xtrans;
		other_y += (int) ytrans;

		// Add points left an right symmetrically.
		added = (CvPoint*)cvSeqPush(seq_right, &nextPoint);
		nextPoint.x = other_x;
		nextPoint.y = other_y;
		added = (CvPoint*)cvSeqPush(seq_left, &nextPoint);
		
		// cout << "t = " << t << " added (";
		// cout << added->x << "," << added->y << ")" << endl;
	}

	// This check was necessary because the gls optimiser produced
	// nonsense parameters as it was not possible to define minimum and
	// maximum values... And if a sequence has no points in it,
	// cvDrawContours() gives only a segmentation fault.
	if (seq_left->total == 0) {
		cerr << endl << "Your parameters did not produce any w-curve points!" << endl;
		exit(0);
	}

	// Store actual pixel width in global variable, because the output
	// comes with the energy function.
	g_w_curve_pixel_width = rightmost*2;

	return;
}


// Retrieve bud contour from input image. We assume that the longest
// contour is the one we want. (Therefore the input image has to be
// white bud on black background, such that the contour of the white
// background is not calculated as contour as well.)
CvSeq* get_bud_contour(IplImage* input_image, CvMemStorage* storage) {

	// Convert input image into binary gray image.
	// Threshold can be modified through slide bar.
	IplImage* gray_image = cvCreateImage(cvGetSize(input_image), 8, 1);
	cvCvtColor(input_image, gray_image, CV_BGR2GRAY);	
	cvThreshold(gray_image, gray_image, g_threshold, 255, CV_THRESH_BINARY);
	
	if (g_show_binary_image) {
		cvResize(gray_image, g_resized_binary_image);
		cvNamedWindow("Binary Image", 1);
		cvMoveWindow("Binary Image", 30, 30);
		cvShowImage("Binary Image", g_resized_binary_image);
	}
	
	
	// Find all contours in the input image and remember how many there
	// are. Use the RETR_LIST method, such that all contours are in one
	// list. Use CHAIN_APPROX_NONE, such that we have points everywhere
	// on the contour. Otherwise parts of the contour with less points
	// would weigh less, which we don't want.
	CvSeq* seq = NULL;
	int num_cont = cvFindContours(gray_image, storage,
		&seq, sizeof(CvContour),
		CV_RETR_LIST,
		CV_CHAIN_APPROX_NONE);

	if (seq == NULL) return NULL;

	// To remember the longest contour and its position in the list,
	// of which seq is the first entry.
	int max_length = 0;
	int max_pos = 0;

	// A new CvSeq pointer because we want to remember seq.
	CvSeq* tmp_seq = seq;

	// Look for the longest contour.
	for (int i=0; i<num_cont; i++) {
		if (tmp_seq->total > max_length) {
			max_length = tmp_seq->total;
			max_pos = i;
		}
		tmp_seq = tmp_seq->h_next;
	}


	// Now go again to the longest contour.
	for (int i=0; i<max_pos; i++) {
		seq = seq->h_next;
	}

	// Calculate the height, width and position of the bud contour as a
	// help for the optimiser. To do this, remember the left-, right-,
	// top- and bottom-most coordinates.
	int leftmost = input_image->width;
	int rightmost = 0;
	int topmost = input_image->height;
	int bottommost = 0;

	for (int i=0; i < seq->total; i++) {
		
		CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq, i);

		if (p->x < leftmost) leftmost = p->x;
		if (p->x > rightmost) rightmost = p->x;

		if (p->y < topmost) topmost = p->y;
		if (p->y > bottommost) bottommost = p->y;
	}

	g_bud_width  = rightmost-leftmost;
	g_bud_height = bottommost-topmost;
	g_bud_pos_x  = leftmost+g_bud_width/2;
	g_bud_pos_y  = topmost+g_bud_height/2;

	//cout << g_bud_width << " " << g_bud_height << " " 
	//cout << g_bud_pos_x << " " << g_bud_pos_y << endl;

	// Release temporary gray_image.
	// Not to be confused with g_gray_image, which is always reused by
	// the energy function.
	cvReleaseImage(&gray_image);

	return seq;
}



// Energy function which calculates how similar the w-curve (given as
// two separate contour sequences for left and right part) and the bud
// contour are.
double contour_energy(CvSeq* contour_w_left, CvSeq* contour_w_right, CvSeq* bud_contour) {


	// If there is no input, return huge_val.
	if ( (contour_w_left == NULL) ||
			(contour_w_right == NULL) ||
			(bud_contour == NULL)
			) return HUGE_VAL;



			
	// Remember the tip and bottom of the contours, because
	// after the new contour recognition, the contour will be
	// in the sequence twice: down and up again. We can still
	// know where it starts by remembering the start and end.
	CvPoint* contour_w_left_top = CV_GET_SEQ_ELEM(CvPoint, contour_w_left, 0);
	CvPoint* contour_w_left_bottom = CV_GET_SEQ_ELEM(CvPoint, contour_w_left, contour_w_left->total - 1);
	CvPoint* contour_w_right_top = CV_GET_SEQ_ELEM(CvPoint, contour_w_right, 0);
	CvPoint* contour_w_right_bottom = CV_GET_SEQ_ELEM(CvPoint, contour_w_right, contour_w_right->total - 1);


	// I must store the values because later I am reusing the storage of the curves... cvClearMemStorage...
	int contour_w_left_top_x = contour_w_left_top->x;
	int contour_w_left_top_y = contour_w_left_top->y;
	int contour_w_left_bottom_x = contour_w_left_bottom->x;
	int contour_w_left_bottom_y = contour_w_left_bottom->y;
	int contour_w_right_top_x = contour_w_right_top->x;
	int contour_w_right_top_y = contour_w_right_top->y;
	int contour_w_right_bottom_x = contour_w_right_bottom->x;
	int contour_w_right_bottom_y = contour_w_right_bottom->y;
	
	// cout << endl;
	// cout << "left top and bottom (" << contour_w_left_top->x << "," << contour_w_left_top->y << ") (" << contour_w_left_bottom->x << "," << contour_w_left_bottom->y << ")" << endl;
	// cout << "right top and bottom (" << contour_w_right_top->x << "," << contour_w_right_top->y << ") (" << contour_w_right_bottom->x << "," << contour_w_right_bottom->y << ")" << endl;
	// cout << endl;
	
	// Draw the contour_w_left white on black.
	cvZero(g_gray_image);
	cvDrawContours(g_gray_image, contour_w_left, cvScalarAll(255), cvScalarAll(255), 0);

	// Debug
	// cout << "Debug 1: " << contour_w_left->total << endl;
	// cvResize(g_gray_image, g_resized_binary_image);
	// cvNamedWindow("Debug 1", 1);
	// cvMoveWindow("Debug 1", 30, 30);
	// cvShowImage("Degbug 1", g_resized_binary_image);
	// cout << "original w_left:" << endl;
	// for (int i=0; i < contour_w_left->total; ++i) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, contour_w_left, i);
		// cout << "(" << p->x << "," << p->y << ")" << "; ";
		// // cvCircle(g_gray_image, *p, 1, CVX_YELLOW, 4);
	// }
    // cout << "\n\n";
	
	
	// Create a new contour to be sure that we have a contour with
	// equidistant points (hence CV-CHAIN-APPROX_NON).
	// Notice: The newly created contour is the original line twice..
	cvClearMemStorage(g_storage_w_left);
	CvSeq* seq_left = NULL;
	int num_cont    = cvFindContours(g_gray_image, g_storage_w_left,
		&seq_left, sizeof(CvContour),
		CV_RETR_LIST,
		CV_CHAIN_APPROX_NONE);

	// If we have no sequence, return huge_val.
	if (seq_left == NULL) return HUGE_VAL;


	// Sometimes more than one contour are found, when the curve touches
	// the border of the image. We only want the longest.
	// To remember the longest contour and its position in the list
	// which seq_left is the first entry of.
	int max_length = 0;
	int max_pos = 0;
	// A new CvSeq pointer because we want to remember seq_left.
	CvSeq* tmp_seq = seq_left;
	// Look for the longest contour.
	for (int i=0; i<num_cont; i++) {
		if (tmp_seq->total > max_length) {
			max_length = tmp_seq->total;
			max_pos = i;
		}
		tmp_seq = tmp_seq->h_next;
	}
	// Go to the longest contour.
	for (int i=0; i<max_pos; i++) {
		seq_left = seq_left->h_next;
	}

	// // Debug
	// cvZero(g_gray_image);
	// cout << "Debug 2: " << seq_left->total << endl;
	// // This shows, that the points are actually twice the same! So it
	// // does not go around as I feared earlier...
	// cout << "processed w_left:" << endl;
	// for (int i=0; i < seq_left->total; ++i) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_left, i);
		// cout << "(" << p->x << "," << p->y << ")" << "; ";
		// // cvCircle(g_gray_image, *p, 1, CVX_YELLOW, 4);
	// }
    // cout << "\n\n";
	// cvDrawContours(g_gray_image, seq_left, cvScalarAll(255), cvScalarAll(255), 0);
	// cvResize(g_gray_image, g_resized_binary_image);
	// cvNamedWindow("Debug 2", 1);
	// cvMoveWindow("Debug 2", 60, 60);
	// cvShowImage("Debug 2", g_resized_binary_image);
	
	
	
	
	
	// Do the same for contour_w_right.
	cvZero(g_gray_image);
	cvDrawContours(g_gray_image, contour_w_right, cvScalarAll(255), cvScalarAll(255), 0);

	// Debug
	// cout << "original w_right:" << endl;
	// for (int i=0; i < contour_w_right->total; ++i) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, contour_w_right, i);
		// cout << "(" << p->x << "," << p->y << ")" << "; ";
		// // cvCircle(g_gray_image, *p, 1, CVX_YELLOW, 4);
	// }
    // cout << "\n\n";
	
	
	
	CvSeq* seq_right = NULL;
	cvClearMemStorage(g_storage_w_right);
	num_cont = cvFindContours(g_gray_image, g_storage_w_right,
		&seq_right, sizeof(CvContour),
		CV_RETR_LIST,
		CV_CHAIN_APPROX_NONE);

	// If we have no sequence, return huge_val.
	if (seq_right == NULL) return HUGE_VAL;

	// Find longest.
	max_length = 0;
	max_pos = 0;
	tmp_seq = seq_right;
	for (int i=0; i<num_cont; i++) {
		if (tmp_seq->total > max_length) {
			max_length = tmp_seq->total;
			max_pos = i;
		}
		tmp_seq = tmp_seq->h_next;
	}
	for (int i=0; i<max_pos; i++) {
		seq_right = seq_right->h_next;
	}
	
	// Debug
	// cout << "processed w_right:" << endl;
	// for (int i=0; i < seq_right->total; ++i) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_right, i);
		// cout << "(" << p->x << "," << p->y << ")" << "; ";
		// // cvCircle(g_gray_image, *p, 1, CVX_YELLOW, 4);
	// }
    // cout << "\n\n";
	
	
	
	// Draw bud contour.
	cvZero(g_gray_image);
	cvDrawContours(g_gray_image, bud_contour, cvScalarAll(255), cvScalarAll(255), 0);

	// Remove the contour pixels on the edges. This is particularly
	// important for the bottom line of the bud contour which should not
	// be considered.
	for (int i=0; i<g_gray_image->width; i++) {
		// top row
		cvSetReal2D(g_gray_image, 0, i, 0.0);
		cvSetReal2D(g_gray_image, 1, i, 0.0);
		// bottom row
		cvSetReal2D(g_gray_image, g_gray_image->height-1, i, 0.0);
		cvSetReal2D(g_gray_image, g_gray_image->height-2, i, 0.0);
	}
	for (int i=0; i<g_gray_image->height; i++) {
		// left column
		cvSetReal2D(g_gray_image, i, 0, 0.0);
		cvSetReal2D(g_gray_image, i, 1, 0.0);
		// right column
		cvSetReal2D(g_gray_image, i, g_gray_image->width-1, 0.0);
		cvSetReal2D(g_gray_image, i, g_gray_image->width-2, 0.0);    
	}

	
	// Invert image2.
	cvNot(g_gray_image, g_gray_image);
	// Perform the distance transformation.
	// (CV_DIST_L1, CV_DIST_L2, CV_DIST_C are also possible)
	cvZero(g_dist_image);
	cvDistTransform(g_gray_image, g_dist_image, CV_DIST_L2, 5);


	// Square the values of the distance image, such that more distant
	// points weight more heavily. (Did not really show any improvement
	// for the optimiser...)
	// cvConvertScale(g_dist_image, g_dist_image, 0.2);
	// cvPow(g_dist_image, g_dist_image, 2);

	
	// Scale. Otherwise you don't see anything, if intermediate output is
	// wanted here.
	if (g_show_dist_image) {
		cvConvertScale(g_dist_image, g_dist_image, 0.01);

		cvResize(g_dist_image, g_resized_dist_image);
		cvNamedWindow("Distance Transformation", 1);
		cvMoveWindow("Distance Transformation", 20, 20);
		cvShowImage("Distance Transformation", g_resized_dist_image);
		
		cvConvertScale(g_dist_image, g_dist_image, 100);
	}
	
	// Go through all points of seq_left and seq_right and see what the
	// respective values in g_dist_image are. These values represent
	// how far they are away from bud_contour.
	double distance_sum = 0.0;

	// Vectors to store the distance of every path-curve-pixel from the
	// measured bud contour.
	vector<double> distances_left;
	vector<double> distances_right;
	// Remember maximum value.
	double distance = 0.0;
	double max_distance = 0.0;
	
	
	int i = 0;
	// Find the top of old seq_left.
	while (i < seq_left->total) {
		CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_left, i);
		i++;
		
		if ((p->x == contour_w_left_top_x) && (p->y == contour_w_left_top_y)) {			
			// cout << "Found top of left at " << i << " of " << seq_left->total << "  (" << p->x << "," << p->y << ")" << endl;
			break;
		}
	}
	if (i == seq_left->total) {
		// cout << "Did not find top of left!! (" << contour_w_left_top_x << "," << contour_w_left_top_y << ")" << endl;
		distance_sum = HUGE_VAL;
		distances_left.clear();
	}
	
	
	// Proceed until the bottom of old seq_left is reached.
	// The bottom can be before the top in the new contour.
	// Hence, we have to look everywhere.
	int count = 0;
	while (count < seq_left->total) {
		CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_left, i);
				
		distance = cvGetReal2D(g_dist_image, p->y, p->x);
		distance_sum += distance*distance;
		
		if (distance > max_distance) {
			max_distance = distance;
		}
		distances_left.push_back(distance);
		
		if ((p->x == contour_w_left_bottom_x) && (p->y == contour_w_left_bottom_y)) {
			//cout << "Found bottom of left at " << i << " of " << seq_left->total << "  (" << p->x << "," << p->y << ")" << endl;
			break;
		}
		
		// Wrap around, when the end is reached, but do not look longer than once.
		i = (i+1) % seq_left->total;
		count++;
	}
	if (count == seq_left->total) {
		// cout << "Did not find bottom of left!! (" << contour_w_left_bottom_x << "," << contour_w_left_bottom_y << ")" << endl;
		distance_sum = HUGE_VAL;
		distances_left.clear();
	}

	
	// The same for right!
	i = 0;
	// Find the top of old seq_right.
	while (i < seq_right->total) {
		CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_right, i);
		i++;
		
		if ((p->x == contour_w_right_top_x) && (p->y == contour_w_right_top_y)) {
			// cout << "Found top of right at " << i << " of " << seq_right->total << "  (" << p->x << "," << p->y << ")" << endl;
			break;
		}
	}
	if (i == seq_right->total) {
		// cout << "Did not find top of right!! (" << contour_w_right_top_x << "," << contour_w_right_top_y << ")" << endl;
		distance_sum = HUGE_VAL;
		distances_right.clear();
	}
	
	
	// Proceed until the bottom of old seq_right is reached.
	// The bottom can be before the top in the new contour.
	// Hence, we have to look everywhere.
	count = 0;
	while (count < seq_right->total) {
		CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, seq_right, i);	
		
		distance = cvGetReal2D(g_dist_image, p->y, p->x);
		distance_sum += distance*distance;
		
		if (distance > max_distance) {
			max_distance = distance;
		}
		distances_right.push_back(distance);
		
		if ((p->x == contour_w_right_bottom_x) && (p->y == contour_w_right_bottom_y)) {
			// cout << "Found bottom of right at " << i << " of " << seq_right->total << "  (" << p->x << "," << p->y << ")" << endl;
			break;
		}
		
		// Wrap around, when the end is reached, but do not look longer than once.
		i = (i+1) % seq_right->total;
		count++;
	}
	if (count == seq_right->total) {
		// cout << "Did not find bottom of right!! (" << contour_w_right_bottom_x << "," << contour_w_right_bottom_y << ")" << endl;
		distance_sum = HUGE_VAL;
		distances_right.clear();
	}
	
 	
	// Draw the output deviation image!
	cvZero(g_deviation_image);
	
	// Draw the axes.
	cvLine(g_deviation_image,
	    cvPoint(0.05*g_deviation_width, 0.9*g_deviation_height),
	    cvPoint(0.95*g_deviation_width, 0.9*g_deviation_height),
	    CVX_RED,
		1);    // thickness

	cvLine(g_deviation_image,
	    cvPoint(0.5*g_deviation_width, 0.1*g_deviation_height),
	    cvPoint(0.5*g_deviation_width, 0.9*g_deviation_height),
		CVX_RED,
		1);    // thickness

	
	// Calculate factors to scale the visualisation. max_distance should be the maximum
	// value on the y-axis, which is g_deviation_height*0.8 high. The values for all pixels
	// shall fit on the x-axis, which is g_deviation_width*0.4 wide on each side.
	double height_factor = (g_deviation_height*0.8) / max_distance;
	double left_width_factor = (g_deviation_width*0.45) / distances_left.size();
	double right_width_factor = (g_deviation_width*0.45) / distances_right.size();
	

	// Draw the distance curves.
	for (unsigned int i=0; i < distances_left.size(); i++) {
		cvCircle(g_deviation_image,
			cvPoint((g_deviation_width*0.5)-(i*left_width_factor), (g_deviation_height*0.9)-(distances_left[i]*height_factor)),
			0, // radius
			CVX_YELLOW,
			1); // thickess
	}


	
	for (unsigned int i=0; i < distances_right.size(); i++) {
		cvCircle(g_deviation_image,
			cvPoint((g_deviation_width*0.5)+(i*right_width_factor), (g_deviation_height*0.9)-(distances_right[i]*height_factor)),
			0, // radius
			CVX_YELLOW,
			1); // thickness
	}


	// Draw the maximum distance next to the y-axis.
	CvFont* myFont = new CvFont();
	cvInitFont(myFont,
		CV_FONT_HERSHEY_PLAIN,
		1.0,  // hscale
		1.0); // vscale


	char max_distance_string[16];
	// TODO: sprintf can cause buffer overflow, if the printed message
	// exceeds the 16 bytes; should not happen...
	sprintf(max_distance_string, "max: %.2f", max_distance);

	
		
	cvPutText(g_deviation_image,
		max_distance_string,
		cvPoint(0.5*g_deviation_width+8, 0.1*g_deviation_height+10),
		myFont,
		CVX_RED);	

		
	// Show deviation image.
	cvShowImage("Deviation", g_deviation_image);
	
	// If there was some error before, don't try the following calculation.
	// (Might result in division by 0...)
	if (distance_sum == HUGE_VAL) {
		return distance_sum;
	}
	else {
		// Return sqrt(sum(x)/n), where x is the square of all distances...
		double return_val = sqrt(distance_sum / ( distances_left.size() +  distances_right.size() ) );
		// Round, otherwise there will be difficulties!
		return_val *= 1000000;
		return_val = round(return_val);
		return_val /= 1000000;
		return return_val;
	}
}


// Function to be executed on trackbar movement.
void on_tb(int) {

	// Recalculate w-curve with global storage.
	CvSeq* contour_w_left = NULL;
	CvSeq* contour_w_right = NULL;


	create_w_curve(((double) g_lambda_tb)/1000,
		// Taking width times lambda will avoid
		// that lambda has an effect on the
		// width.
		g_width_tb * (((double) g_lambda_tb)/1000),
		g_height_tb,
		g_rot_tb,
		g_xtrans_tb, g_ytrans_tb,
		contour_w_left,
		contour_w_right,
		g_storage_w_left,
		g_storage_w_right);
	
	
	// Reset output image.
	cvZero(g_output_image);
	// Show input image in background of output image.
	cvAddWeighted(g_output_image, 1.0 - (double) g_input_alpha / 100,
		g_input_image, (double) g_input_alpha / 100,
		0.0, g_output_image);


	// Draw contours.
	cvDrawContours(g_output_image, contour_w_left, CVX_RED, CVX_RED, 0);
	cvDrawContours(g_output_image, contour_w_right, CVX_RED, CVX_RED, 0);
	cvDrawContours(g_output_image, g_contour_bud, CVX_YELLOW, CVX_YELLOW, 0);


	// Store energy in global variable to be returned upon exit.
	g_energy = contour_energy(contour_w_left, contour_w_right, g_contour_bud);
	cout << "energy=" << g_energy;
	
	
	// Draw contours bold for printout.
	// for (int i=0; i < contour_w_left->total; i++) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, contour_w_left, i);
		// cvCircle(g_output_image, *p, 1, CVX_RED, 2);
	// }
	// for (int i=0; i < contour_w_right->total; i++) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, contour_w_right, i);
		// cvCircle(g_output_image, *p, 1, CVX_RED, 2);
	// }	
	// for (int i=0; i < g_contour_bud->total; i++) {
		// CvPoint* p = CV_GET_SEQ_ELEM(CvPoint, g_contour_bud, i);
		// cvCircle(g_output_image, *p, 1, CVX_YELLOW, 2);
	// }

	
	// Resize to fit the screen dimensions.
	cvResize(g_output_image, g_resized_image);
	cvShowImage("Output", g_resized_image);
		
	
	cout << ", lambda=" << ((double)g_lambda_tb)/1000 << ", ";
	cout << "w=" << g_width_tb * (((double) g_lambda_tb)/1000);
	cout << " (pixel-w=" << g_w_curve_pixel_width << "), ";
	cout << "h=" << g_height_tb;

	cout << endl;

	return;
}


// If theshold trackbar is moved recalculate the bud contour.
void on_tb_thresh(int) {

	// Get the bud contour.
	g_contour_bud = get_bud_contour(g_input_image, g_storage_bud);

	on_tb(0);
	return;
}


// A function to check if the altered rotation values are still within
// the given range.
bool degree_ok(int degree, int min, int max) {

	assert(degree<=360);

	if (min < max) {
		if ((degree > min) && (degree < max)) return true;
		else return false;
	}

	// The range is around 0°.
	else if (min > max) {
		if ((degree > min) || (degree < max)) return true;
		else return false;
	}

	else return false;
}


// A function which adds values to the given vector recursively.
void add_values(// The 6-value vector which the alteration is
// added to.
const vector<double> original,
// The index of the parameter which was changed in
// "original", because we only alter the parameters
// with greater indices. The combinations with smaller
// indices are already covered by previous invocations
// of this function.
int index,
// The depth of how many recursions are still to be done.
int recursions,
// The current delta.
double delta,
// Arrays of current parameter values, maxima, minima
// an factors.
double *params,
double *max,
double *min,
double *factors,
// The vector which the new values are added to.
vector< vector<double> > &values) {


	// Create a vector with 6 entries which are all 0.
	vector<double> next_try;

	--recursions;

	// Fill "values" with alteration possibilities.
	for (int i=index; i<6; i++) {
		// Special treatment for rotation angle.
		if (i==3) {
			if (degree_ok(add_degree(params[i], original[i]+factors[i]*delta),
						min[i], max[i]
						)) {
				next_try = original; // Copy vector content.
				next_try[i] += factors[i]*delta;
				values.push_back(next_try);
				
				if(recursions>0) {
					add_values(next_try, i+1, recursions, delta,
					params, max, min, factors,
					values);
				}
			}
			if (degree_ok(add_degree(params[i], original[i]-factors[i]*delta),
						min[i], max[i]
						)) {
				next_try = original;  // Copy vector content.
				next_try[i] -= factors[i]*delta;
				values.push_back(next_try);

				if(recursions>0) {
					add_values(next_try, i+1, recursions, delta,
					params, max, min, factors,
					values);
				}
			}
		}
		// Other paramaters which have proper max and min.
		else {
			if (original[i]+factors[i]*delta+params[i] < max[i]) {

				//cout << original[i]+factors[i]*delta+params[i] << " <= " << max[i] << endl;

				next_try = original; // Copy vector content.
				next_try[i] += factors[i]*delta;
				values.push_back(next_try);

				if(recursions>0) {
					add_values(next_try, i+1, recursions, delta,
					params, max, min, factors,
					values);
				}
			}
			if (original[i]-factors[i]*delta+params[i] > min[i]) {

				//cout << original[i]-factors[i]*delta+params[i] << " >= " << min[i] << endl;

				next_try = original; // Copy vector content.
				next_try[i] -= factors[i]*delta;
				values.push_back(next_try);

				if(recursions>0) {
					add_values(next_try, i+1, recursions, delta,
					params, max, min, factors,
					values);
				}
			}
		}
	}
}


// Optimiser using the best neighbour search.
void optimiser(double lambda_min,
	double lambda_max,
	double width_min,
	double width_max,
	double height_min,
	double height_max,
	double rot_min,
	double rot_max,
	double xtrans_min,
	double xtrans_max,
	double ytrans_min,
	double ytrans_max,
	int recursions		 
	) {

	cout << "Optimising..." << endl;


	// Vectors to access the values by indices.
	double min[6] = {lambda_min,
		width_min,
		height_min,
		rot_min,
		xtrans_min,
		ytrans_min};
	
	double max[6] = {lambda_max,
		width_max,
		height_max,
		rot_max,     
		xtrans_max,
		ytrans_max};


	// Convert the trackbar values, which are integer, into doubles.
	double lambda = (double) g_lambda_tb;
	double width  = (double) g_width_tb;
	double height = (double) g_height_tb;
	double rot    = (double) g_rot_tb;
	double xtrans = (double) g_xtrans_tb;
	double ytrans = (double) g_ytrans_tb;

	// Array for index access.
	double params[6] = {lambda,
		width,
		height,
		rot,
		xtrans,
		ytrans};

	// Factors to multiply the [0,1.0]-delta with.
	double lambda_factor = lambda_max - lambda_min;
	double width_factor  = width_max - width_min;
	double height_factor = height_max - height_min;
	double rot_factor    = rot_max - rot_min;
	double xtrans_factor = xtrans_max - xtrans_min;
	double ytrans_factor = ytrans_max - ytrans_min;

	// Array such that the values can be accessed through their indices.
	double factors[6] = {lambda_factor,
		width_factor,
		height_factor,
		rot_factor,
		xtrans_factor,
		ytrans_factor};
	

	// Recalculate w-curve with global storage.
	CvSeq* contour_w_left = NULL;
	CvSeq* contour_w_right = NULL;

	create_w_curve(lambda/1000,
		// Taking width times lambda will avoid
		// that lambda has an effect on the
		// width.
		width * (lambda/1000),
		height,
		rot,
		xtrans, ytrans,
		contour_w_left,
		contour_w_right,
		g_storage_w_left,
		g_storage_w_right);


	double current_energy = contour_energy(contour_w_left, contour_w_right, g_contour_bud);

	// Variable to store new energy.
	double new_energy = current_energy;

	// Variable to indicate which variation was the best neighbour.
	// -1 will tell us, that no better neighbour has been found.
	int number_of_best;

	// A vector containing all the test values, which are themselves
	// each a vector consisting of 6 elements, which give the value by
	// which the respective parameter is to be changed.
	vector< vector<double> > values;

	// A first empty 6-value vector.
	vector<double> zero_vec;

	// Start with a relatively great delta
	double delta = 0.5;
	
	// While delta is still great keep optimising.
	while (delta > 0.005) {
	
		// Set to 0, otherwise the while loop won't start...
		number_of_best = 0;
		while (number_of_best > -1) {
			
			// Clear vector.
			values.clear();
			
			zero_vec.assign(6, 0.0);
			add_values(
				zero_vec,
				0, // Index
				recursions, // Recursions
				delta,
				params,
				max,
				min,
				factors,
				values
			);

			// Will be >-1, if a better neighbour is found in the follwoing loop.
			number_of_best = -1;

			//cout << "Current energy is " << current_energy << endl;

			// Try all the entries of "values" and see which is the best.
			for (unsigned int i=0; i < values.size(); i++) {
				
				create_w_curve(
					(params[0]+values[i][0])/1000,
					// Taking width times lambda will avoid
					// that lambda has an effect on the
					// width.
					(params[1]+values[i][1]) * ((params[0]+values[i][0])/1000),
					params[2]+values[i][2],
					add_degree(params[3],values[i][3]),
					params[4]+values[i][4],
					params[5]+values[i][5],
					contour_w_left,
					contour_w_right,
					g_storage_w_left,
					g_storage_w_right
				);
				
				new_energy = contour_energy(contour_w_left, contour_w_right, g_contour_bud);
								
				// If the new curve yields a better energy, remember it.
				if (new_energy < current_energy) {
					current_energy = new_energy;
					number_of_best = i;
				}
			}



			// Found a better neighbour?
			if (number_of_best > -1) {

				//cout << number_of_best << " is best." << endl;
				
				for (int i=0; i<6; i++) {
					if (i==3) {
						params[i] = add_degree(params[i],values[number_of_best][i]);
					}
					else {
						params[i] += values[number_of_best][i];
					}
				}
			}
		}

		// Reduce delta.
		delta /= 1.6;
	}


	g_lambda_tb = (int) params[0];
	g_width_tb  = (int) params[1];
	g_height_tb = (int) params[2];
	g_rot_tb    = (int) params[3];
	g_xtrans_tb = (int) params[4];
	g_ytrans_tb = (int) params[5];

	// Move trackbars to respective positions.
	cvSetTrackbarPos("Lambda*1000", "Trackbars", g_lambda_tb);
	cvSetTrackbarPos("Width", "Trackbars", g_width_tb);
	cvSetTrackbarPos("Height", "Trackbars", g_height_tb);
	cvSetTrackbarPos("Rotation", "Trackbars", g_rot_tb);
	cvSetTrackbarPos("Translation x", "Trackbars", g_xtrans_tb);
	cvSetTrackbarPos("Translation y", "Trackbars", g_ytrans_tb);

	on_tb(0);
	
	cout << "...optimisation done!" << endl;
}


// Adjust w-curve width such that the actual pixel width of the
// w-curve matches the contour width.
void adjust_w_curve_width(void) {
	// Make lambda variable for better readability.
	double lambda = ((double)g_lambda_tb)/1000;      

	g_width_tb = (int)
		(g_bud_width 
		* (exp(-lambda*g_rightmost_t) +
		(g_height_tb/2)*exp(g_rightmost_t))
		) / (2*lambda);

	cvSetTrackbarPos("Width", "Trackbars", g_width_tb);
}

// Adjust w-curve height according to bud contour height.
void adjust_w_curve_height(void) {
	g_height_tb = g_bud_height;
	cvSetTrackbarPos("Height", "Trackbars", g_height_tb);
}

// Center w-curve according to bud contour coordinates.
void adjust_w_curve_pos(void) {
	g_xtrans_tb = g_bud_pos_x;
	g_ytrans_tb = g_bud_pos_y;
	cvSetTrackbarPos("Translation x", "Trackbars", g_xtrans_tb);
	cvSetTrackbarPos("Translation y", "Trackbars", g_ytrans_tb);
}

// Reset Lambda to 1.
void adjust_w_curve_lambda(void) {
	g_lambda_tb = 1000;
	cvSetTrackbarPos("Lambda*1000", "Trackbars", g_lambda_tb);
}

void adjust_w_curve_all(void) {
	adjust_w_curve_pos();
	adjust_w_curve_height();
	adjust_w_curve_lambda();
	// Needs to be last, because it depends on lambda and height.
	adjust_w_curve_width();
}


// Initialising, assuming g_input_image is set.
void initialise() {


	int img_w = cvGetSize(g_input_image).width;
	int img_h = cvGetSize(g_input_image).height;


	// Parameters for the w-curve. The following initial values give a
	// "decent" curve...
	g_width_tb   = 6*img_w;
	g_height_tb  = (3*img_h)/4;
	g_xtrans_tb  = img_w/2;
	g_ytrans_tb  = img_h/2;

	// Initialise global images and storage for the energy function's
	// swap variables.
	cvReleaseImage(&g_gray_image);
	cvReleaseImage(&g_dist_image);
	g_gray_image = cvCreateImage(cvSize(img_w, img_h), 8, 1);
	g_dist_image = cvCreateImage(cvSize(img_w, img_h), IPL_DEPTH_32F, 1);


	cvReleaseImage(&g_resized_image);
	cvReleaseImage(&g_resized_dist_image);
	cvReleaseImage(&g_resized_binary_image);


	
	// If the height/width ratio is greater than 6/8 resize height to 600.
	if (img_h > (img_w*6)/8) {
		g_resized_image = cvCreateImage(cvSize((img_w*600)/img_h, 600), // size
			8,                    // depth
			3);                   // channels
		
		// We also want to give the distance transformation as a visualisation.
		// For this we also need a resized image, in which the original can be
		// resized.
		g_resized_dist_image = cvCreateImage(cvSize((img_w*600)/img_h, 600), // size
			IPL_DEPTH_32F,        // depth
			1);                   // channels
		g_resized_binary_image = cvCreateImage(cvSize((img_w*600)/img_h, 600),
			8,
			1);
		
		// This flag becomes important for registering the correct mouse click positions
		// in the resized windows.
		g_resized_width = false;
	}
	// else adjust width to 800.
	else {
		g_resized_image = cvCreateImage(cvSize(800, (img_h*800)/img_w), // size
			8,                    // depth
			3);                   // channels
		g_resized_dist_image = cvCreateImage(cvSize(800, (img_h*800)/img_w), // size
			IPL_DEPTH_32F,        // depth
			1);                   // channels
		g_resized_binary_image = cvCreateImage(cvSize(800, (img_h*800)/img_w),
			8,
			1);
		
		g_resized_width = true;
	}
	
	
	cvReleaseImage(&g_output_image);
	// Create a colour image for output.
	g_output_image = cvCreateImage(cvSize(img_w, img_h), // size
		8,                    // depth
		3);                   // channels
	
	// Global storage for the w-curve sequence.
	// Needs to be global because it is altered all the time.
	cvReleaseMemStorage(&g_storage_w_left);
	cvReleaseMemStorage(&g_storage_w_right);
	g_storage_w_left = cvCreateMemStorage(0);
	g_storage_w_right = cvCreateMemStorage(0);
	
	// Storage for the bud contour , which will actually be used for
	// more than just the contour which we are interested in but this
	// does not matter. We only want the biggest contour.
	cvReleaseMemStorage(&g_storage_bud);
	g_storage_bud = cvCreateMemStorage(0);

	// Get the bud contour.
	g_contour_bud = get_bud_contour(g_input_image, g_storage_bud);
	
	
	
	
	// Create the windows.
	cvNamedWindow("Output", 1);
	cvMoveWindow("Output", 0, 0);

	// Trackbars for manual curve adjustment...
	cvNamedWindow("Trackbars", 1);
	cvResizeWindow("Trackbars", 400, 600);
	cvMoveWindow("Trackbars", 800+8, 0);
		
	cvCreateTrackbar("Lambda*1000", "Trackbars", &g_lambda_tb, 6000, on_tb);
	cvCreateTrackbar("Width", "Trackbars", &g_width_tb, img_w*20, on_tb);
	cvCreateTrackbar("Height", "Trackbars", &g_height_tb, img_h, on_tb);
	cvCreateTrackbar("Rotation", "Trackbars", &g_rot_tb, 360, on_tb);
	cvCreateTrackbar("Translation x", "Trackbars", &g_xtrans_tb, img_w, on_tb);
	cvCreateTrackbar("Translation y", "Trackbars", &g_ytrans_tb, img_h, on_tb);
	cvCreateTrackbar("Cut-off Top", "Trackbars", &g_cutoff_top, 49, on_tb);
	cvCreateTrackbar("Cut-off Bottom", "Trackbars", &g_cutoff_bottom, 49, on_tb);
	cvCreateTrackbar("Show input", "Trackbars", &g_input_alpha, 100, on_tb);
	cvCreateTrackbar("Threshold", "Trackbars", &g_threshold, 250, on_tb_thresh);



	cvNamedWindow("Deviation", 1);
	cvMoveWindow("Deviation", 0, 625);
	
	
	on_tb(0);
}


// A mouse callback function to read the top and bottom coordinate for
// the w-curve.
void on_mouse_read_coordinates(int event, int x, int y, int flags, void* param) {

	// Rescale the coordinates according to the resizes image, which is shown
	// in the window.
	double factor = 0.0;
	if (g_resized_width) {
		factor = (double) cvGetSize(g_input_image).width / 800;
	}
	else {
		factor = (double) cvGetSize(g_input_image).height / 600;
	}
		
	x = x * factor;
	y = y * factor;

	// Only react to left click.
	if (!(event == CV_EVENT_LBUTTONDOWN)) return;

	else if (g_read_top) {
		cout << "Top is at: (" << x << "," << y << ")" << endl;

		g_mouse_set_top_x = x;
		g_mouse_set_top_y = y;

		g_read_top = false;
		return;
	}

	else if (g_read_bottom) {
		cout << "Bottom is at: (" << x << "," << y << ")" << endl;

		// Calculate the center coordinates of the w-curve.
		g_xtrans_tb = g_mouse_set_top_x + (x - g_mouse_set_top_x)/2;
		g_ytrans_tb = g_mouse_set_top_y + (y - g_mouse_set_top_y)/2;

		// Calculate the height of the w-curve (Pythagoras).
		g_height_tb = sqrt(((x - g_mouse_set_top_x)*(x - g_mouse_set_top_x))+
		((y - g_mouse_set_top_y)*(y - g_mouse_set_top_y)));

		// Calculate sin and cos of the angle. (The w-curve height
		// corrensponds to 1 on the Einheitskreis...)
		double my_sin = (double)(y - g_mouse_set_top_y) / g_height_tb;

		double rad = asin(my_sin);

		// Now we have to see, in which quadrant of the Einheitskreis we
		// are. But everything is shifted 45° counterclockwise and the degrees
		// are counted clockwise...
		// 3rd or 4th
		if (y-g_mouse_set_top_y < 0) {
			// 3rd
			if (g_mouse_set_top_x-x < 0) {
				rad = M_PI - rad;
			}
			// 4th
			else {
				rad = 2*M_PI + rad;
			}
			rad = 5*M_PI/2 - rad;
		}
		// 2nd
		else if (g_mouse_set_top_x-x < 0) {
			rad = M_PI - rad;
			rad = 5*M_PI/2 - rad;
		}
		// 1st
		else {
			rad = M_PI/2 - rad;
		}

		g_rot_tb = rad*180/M_PI;

		cvSetTrackbarPos("Translation x", "Trackbars", g_xtrans_tb);
		cvSetTrackbarPos("Translation y", "Trackbars", g_ytrans_tb);
		cvSetTrackbarPos("Height", "Trackbars", g_height_tb);
		cvSetTrackbarPos("Rotation", "Trackbars", g_rot_tb);

		g_read_bottom = false;
	}

	return;
}

void read_mouse_coordinates(void) {
	g_read_top    = true;
	g_read_bottom = true;
	// Set the mouse callback function.
	cvSetMouseCallback("Output", on_mouse_read_coordinates);

	cout << "You can click the coordinates for the lambda curve's top and bottom. First top, then bottom." << endl;

}


// Cut a region of interest from g_input_image.
// The values are stored globally.
void cut_roi() {
	
	// If the ROI from a previously viewed image is outside the range of
	// this image, reset ROI and do nothing!
	if ((g_roi_top_x + g_roi_width > cvGetSize(g_input_image).width) ||
		(g_roi_top_y + g_roi_height > cvGetSize(g_input_image).height) ) {
		
		g_roi_width = 0;
		g_roi_height = 0;
		return;
	}	
	
	// If there is nothing to crop, return. This is also a way of
	// showing that the whole image should be shown.
	if ((g_roi_width == 0) || (g_roi_height == 0)) {
		return;
	}

	// Create destination image.
	g_cropped_image = cvCreateImage(cvSize(g_roi_width, g_roi_height),
		g_input_image->depth,
		g_input_image->nChannels);

	// Set the Region of interest in the source image.
	//cout << "Setting " << g_roi_top_x << " " << g_roi_top_y << " " << g_roi_width << " " << g_roi_height << endl;
	cvSetImageROI(g_input_image, cvRect(g_roi_top_x, g_roi_top_y, g_roi_width, g_roi_height));
	
	// Copy from source into destination.
	cvCopy(g_input_image, g_cropped_image);
	
	cvReleaseImage(&g_input_image);
	g_input_image = g_cropped_image;
	g_cropped_image = NULL;
	
}



// A mouse callback function to read the top and bottom coordinate for
// the w-curve.
void on_mouse_read_cut(int event, int x, int y, int flags, void* param) {

	// Rescale the coordinates according to the resizes image, which is shown
	// in the window.
	double factor = 0.0;
	if (g_resized_width) {
		factor = (double) cvGetSize(g_input_image).width / 800;
	}
	else {
		factor = (double) cvGetSize(g_input_image).height / 600;
	}

	x = x * factor;
	y = y * factor;

	// Only react to left click.
	if (!(event == CV_EVENT_LBUTTONDOWN)) return;

	else if (g_read_top) {
		cout << "First corner is at: (" << x << "," << y << ")" << endl;

		g_roi_top_x = x;
		g_roi_top_y = y;

		g_read_top = false;
		return;
	}

	else if (g_read_bottom) {
		cout << "Second corner is at: (" << x << "," << y << ")" << endl;

		// Now we know the area which we want to cut from the image.
		// Calculate the dimensions of the destination image.
		g_roi_height = 0;
		g_roi_width = 0;
		// Make sure g_mouse_set_top_x stores the smaller
		// (i.e. more upper left) x value.
		if (x - g_roi_top_x < 0) {
			g_roi_width = g_roi_top_x - x;
			g_roi_top_x = x;
		}
		else if (g_roi_top_x - x < 0) {
			g_roi_width = x - g_roi_top_x;
		}
		
		// Same for height and y.
		if (y - g_roi_top_y < 0) {
			g_roi_height = g_roi_top_y - y;
			g_roi_top_y = y;
		}
		else if (g_roi_top_y - y < 0) {
			g_roi_height = y - g_roi_top_y;
		}
		
		// Do the cropping.
		cut_roi();
		// Restart with new image.	
		initialise();
				
		g_read_bottom = false;
	}

	return;
}


void read_mouse_cut(void) {
	g_read_top    = true;
	g_read_bottom = true;
	// Set the mouse callback function.
	cvSetMouseCallback("Output", on_mouse_read_cut);

	cout << "You can click the two corners of your region of interest." << endl;
}


int go_fit_W(const char* arg,
	int &thresh_inout,
	double &lambda_out,
	double &energy_out,
	int &roi_x_inout,
	int &roi_y_inout,
	int &roi_width_inout,
	int &roi_height_inout,
	int &cutoff_top,
	int &cutoff_bottom
	) {

	// Input image of the bud.
	cvReleaseImage(&g_input_image);
	g_input_image = cvLoadImage(arg);

	g_deviation_width = 600;
	g_deviation_height = 100;
	
	
	// Image to output the w-curve-deviation diagram!
	cvReleaseImage(&g_deviation_image);
	g_deviation_image = cvCreateImage(cvSize(g_deviation_width, g_deviation_height),     // size
                                    8,                        // depth
                                    3);                       // channels

	
	
	
	// Initial threshold value.
	g_threshold = thresh_inout;
	
	g_lambda_tb = lambda_out * 1000;

	g_cutoff_top = cutoff_top;
	g_cutoff_bottom = cutoff_bottom;	

	g_rot_tb = 325;
	g_input_alpha = 90;

	// Take the cropping values from the GUI.
	g_roi_top_x = roi_x_inout;
	g_roi_top_y = roi_y_inout;
	g_roi_width = roi_width_inout;
	g_roi_height = roi_height_inout;
	
	
	// Do the cropping.
	cut_roi();

	
	// Show the windows.
	initialise();


	// Do not react to mouse clicks unless wanted by read_mouse_coordinates().
	g_read_top = false;
	g_read_bottom = false;

	
	// "Menu"
	bool exit = false;
	char c = 0;
	while(!exit) {
		c = cvWaitKey(0);

		// Set lambda curve coordinates with mouse.
		if (c == 'm') {
			read_mouse_coordinates();
		}
		// Select the area to cut from the image, which should be used
		// as new image.
		if (c == 's') {
			read_mouse_cut();
		}
		// Undo the crop selection.
		if (c == 'q') {
			g_roi_width = 0;
			g_roi_height = 0;
			cvReleaseImage(&g_input_image);
			g_input_image = cvLoadImage(arg);
			initialise();
		}
		
		if (c == 'o') {
			optimiser(// Ranges of parameters which make sense.
				200,  // lambda_min
				6000, // lambda_max
				// TODO: Mach das doch mal von der Knospenform abhängig.
				cvGetSize(g_input_image).width * 2,      // width_min
				cvGetSize(g_input_image).width * 20,     // width_max
				cvGetSize(g_input_image).height / 10,    // height_min
				(9*cvGetSize(g_input_image).height) / 10,// height_max
				270,  // rot_min
				90,   // rot_max
				cvGetSize(g_input_image).width / 5,      // xtrans_min
				(4*cvGetSize(g_input_image).width) / 5,  // xtrams_max
				cvGetSize(g_input_image).height / 5,     // ytrans_min
				(4*cvGetSize(g_input_image).height) / 5, // ytrans_max
				1    // recursion depth
			);
		}
		
		else if (c == 'p') {
			optimiser(// Optimise only width.
				g_lambda_tb,  // lambda_min
				g_lambda_tb,  // lambda_max
				cvGetSize(g_input_image).width * 2,      // width_min
				cvGetSize(g_input_image).width * 20,     // width_max
				g_height_tb,    // height_min
				g_height_tb,    // height_max
				g_rot_tb,
				g_rot_tb,
				g_xtrans_tb,    // xtrans_min
				g_xtrans_tb,    // xtrams_max
				g_ytrans_tb,    // ytrans_min
				g_ytrans_tb,    // ytrans_max
				1    // recursion depth
			);

			optimiser(// Optimise only lambda.
				900,  // lambda_min
				1300,  // lambda_max
				g_width_tb*(0.8),     // width_min
				g_width_tb*(1.2),     // width_max
				g_height_tb,    // height_min
				g_height_tb,    // height_max
				g_rot_tb,
				g_rot_tb,
				g_xtrans_tb,    // xtrans_min
				g_xtrans_tb,    // xtrams_max
				g_ytrans_tb,    // ytrans_min
				g_ytrans_tb,    // ytrans_max
				1    // recursion depth
			);
			
			optimiser(// Optimise all in small range only.
				g_lambda_tb*(0.95),  // lambda_min
				g_lambda_tb*(1.05), // lambda_max
				g_width_tb*(0.95),      // width_min
				g_width_tb*(1.05),     // width_max
				g_height_tb*(0.95),    // height_min
				g_height_tb*(1.05),    // height_max
				g_rot_tb-5,
				g_rot_tb+5,
				g_xtrans_tb*(0.95),    // xtrans_min
				g_xtrans_tb*(1.05),    // xtrams_max
				g_ytrans_tb*(0.95),    // ytrans_min
				g_ytrans_tb*(1.05),    // ytrans_max
				1    // recursion depth
			);


		}
		else if (c == 'u') {
			optimiser(// Optimise only width.
				g_lambda_tb,  // lambda_min
				g_lambda_tb,  // lambda_max
				cvGetSize(g_input_image).width * 2,      // width_min
				cvGetSize(g_input_image).width * 20,     // width_max
				g_height_tb,    // height_min
				g_height_tb,    // height_max
				g_rot_tb,
				g_rot_tb,
				g_xtrans_tb,    // xtrans_min
				g_xtrans_tb,    // xtrams_max
				g_ytrans_tb,    // ytrans_min
				g_ytrans_tb,    // ytrans_max
				1    // recursion depth
			);
		}     
		
		else if (c == 'i') {
			optimiser(// Optimise only lambda.
				900,  // lambda_min
				1300,  // lambda_max
				g_width_tb*(0.8),      // width_min
				g_width_tb*(1.2),     // width_max
				g_height_tb,    // height_min
				g_height_tb,    // height_max
				g_rot_tb,
				g_rot_tb,
				g_xtrans_tb,    // xtrans_min
				g_xtrans_tb,    // xtrams_max
				g_ytrans_tb,    // ytrans_min
				g_ytrans_tb,    // ytrans_max
				1    // recursion depth
			);
			
		}
		
		else if (c == 'w') {
			adjust_w_curve_width();
		}
		else if (c == 'h') {
			adjust_w_curve_height();
		}
		else if (c == 't') {
			adjust_w_curve_pos();
		}
		else if (c == 'l') {
			adjust_w_curve_lambda();
		}
		else if (c == 'r') {
			adjust_w_curve_all();
		}

		// Exit when a escape key is pressed.
		else if (c == 27) {
			lambda_out = 9999;
			exit = true;
		}
		
		// Exit when a enter key is pressed and pass the values.
		else if (c == 13) {
			
			// Return the current threshold to the caller.
			thresh_inout = g_threshold;
			lambda_out = (double) g_lambda_tb / 1000;
			energy_out = g_energy;
			roi_x_inout = g_roi_top_x;
			roi_y_inout = g_roi_top_y;
			roi_width_inout = g_roi_width;
			roi_height_inout = g_roi_height;
			cutoff_top = g_cutoff_top;
			cutoff_bottom = g_cutoff_bottom;
			
			exit = true;
		}
		
		else if (c == 'd') {
			g_show_dist_image = !g_show_dist_image;
			cvDestroyWindow("Distance Transformation");
			on_tb(0);
		}
		else if (c == 'b') {
			g_show_binary_image = !g_show_binary_image;
			cvDestroyWindow("Binary Image");
			on_tb_thresh(0);
		}
		
	}

	cvDestroyAllWindows();

	// Release global images and storage of energy function.
	cvReleaseImage(&g_gray_image);
	cvReleaseImage(&g_dist_image);
	cvReleaseImage(&g_resized_image);
	cvReleaseImage(&g_input_image);
	cvReleaseImage(&g_output_image);
	
	cvReleaseImage(&g_resized_dist_image);
	cvReleaseImage(&g_resized_binary_image);
	
	cvReleaseMemStorage(&g_storage_bud);
	cvReleaseMemStorage(&g_storage_w_left);
	cvReleaseMemStorage(&g_storage_w_right);

	return 0;
}
